@@ -1,4 +1,4 @@
-/* p_exe.cpp --
+/* p_exe.cpp -- dos/exe executable format
 
    This file is part of the UPX executable compressor.
 
@@ -35,9 +35,8 @@
 static const CLANG_FORMAT_DUMMY_STATEMENT
 #include "stub/i086-dos16.exe.h"
 
-#define RSFCRI 4096 // Reserved Space For Compressed Relocation Info
 #define MAXMATCH 0x2000
-#define MAXRELOCS (0x8000 - MAXMATCH)
+#define MAXRELOCSIZE (0x8000 - MAXMATCH)
 
 #define DI_LIMIT 0xff00 // see the assembly why
 
@@ -49,16 +48,14 @@ PackExe::PackExe(InputFile *f) : super(f) {
     bele = &N_BELE_RTP::le_policy;
     COMPILE_TIME_ASSERT(sizeof(exe_header_t) == 32)
     COMPILE_TIME_ASSERT_ALIGNED1(exe_header_t)
-    ih_exesize = ih_imagesize = ih_overlay = 0;
-    stack_for_lzma = 0;
-    use_clear_dirty_stack = false;
 }
 
+Linker *PackExe::newLinker() const { return new ElfLinkerX86(); }
+
 const int *PackExe::getCompressionMethods(int method, int level) const {
     bool small = ih_imagesize <= 256 * 1024;
     // disable lzma for "--brute" unless explicitly given "--lzma"
-    // WARNING: this side effect may persists for later files;
-    //   but note that class PackMaster creates per-file local options
+    // (note that class PackMaster creates per-file local options)
     if (opt->all_methods_use_lzma == 1 && !opt->method_lzma_seen)
         opt->all_methods_use_lzma = 0;
     return Packer::getDefaultCompressionMethods_8(method, level, small);
@@ -75,7 +72,7 @@ int PackExe::fillExeHeader(struct exe_header_t *eh) const {
     if (ih.relocs == 0)
         flag |= NORELOC;
 
-    memset(&oh, 0, sizeof(oh));
+    mem_clear(&oh);
     oh.ident = 'M' + 'Z' * 256;
     oh.headsize16 = 2;
 
@@ -106,16 +103,16 @@ int PackExe::fillExeHeader(struct exe_header_t *eh) const {
 }
 
 void PackExe::addLoaderEpilogue(int flag) {
-    addLoader("EXEMAIN5", nullptr);
+    addLoader("EXEMAIN5");
     if (relocsize)
         addLoader(ph.u_len <= DI_LIMIT || (ph.u_len & 0x7fff) >= relocsize ? "EXENOADJ"
                                                                            : "EXEADJUS",
                   "EXERELO1", has_9a ? "EXEREL9A" : "", "EXERELO2",
-                  ih_exesize > 0xFE00 ? "EXEREBIG" : "", "EXERELO3", nullptr);
+                  ih_exesize > 0xFE00 ? "EXEREBIG" : "", "EXERELO3");
     addLoader("EXEMAIN8", device_driver ? "DEVICEEND" : "", (flag & SS) ? "EXESTACK" : "",
-              (flag & SP) ? "EXESTASP" : "", (flag & USEJUMP) ? "EXEJUMPF" : "", nullptr);
+              (flag & SP) ? "EXESTASP" : "", (flag & USEJUMP) ? "EXEJUMPF" : "");
     if (!(flag & USEJUMP))
-        addLoader(ih.cs ? "EXERCSPO" : "", "EXERETIP", nullptr);
+        addLoader(ih.cs ? "EXERCSPO" : "", "EXERETIP");
 
     linker->defineSymbol("original_cs", ih.cs);
     linker->defineSymbol("original_ip", ih.ip);
@@ -123,7 +120,7 @@ void PackExe::addLoaderEpilogue(int flag) {
     linker->defineSymbol("original_ss", ih.ss);
     linker->defineSymbol(
         "reloc_size",
-        (ph.u_len <= DI_LIMIT || (ph.u_len & 0x7fff) >= relocsize ? 0 : MAXRELOCS) - relocsize);
+        (ph.u_len <= DI_LIMIT || (ph.u_len & 0x7fff) >= relocsize ? 0 : MAXRELOCSIZE) - relocsize);
 }
 
 void PackExe::buildLoader(const Filter *) {
@@ -136,7 +133,7 @@ void PackExe::buildLoader(const Filter *) {
     if (M_IS_LZMA(ph.method)) {
         addLoader("LZMA_DEC00", opt->small ? "LZMA_DEC10" : "LZMA_DEC20", "LZMA_DEC30",
                   use_clear_dirty_stack ? "LZMA_DEC31" : "", "LZMA_DEC32",
-                  ph.u_len > 0xffff ? "LZMA_DEC33" : "", nullptr);
+                  ph.u_len > 0xffff ? "LZMA_DEC33" : "");
 
         addLoaderEpilogue(flag);
         defineDecompressorSymbols();
@@ -173,35 +170,35 @@ void PackExe::buildLoader(const Filter *) {
         initLoader(stub_i086_dos16_exe, sizeof(stub_i086_dos16_exe));
         // prepare loader
         if (device_driver)
-            addLoader("DEVICEENTRY,LZMADEVICE,DEVICEENTRY2", nullptr);
+            addLoader("DEVICEENTRY,LZMADEVICE,DEVICEENTRY2");
 
         linker->addSection("COMPRESSED_LZMA", compressed_lzma, c_len_lzma, 0);
-        addLoader("LZMAENTRY,NRV2B160,NRVDDONE,NRVDECO1,NRVGTD00,NRVDECO2", nullptr);
+        addLoader("LZMAENTRY,NRV2B160,NRVDDONE,NRVDECO1,NRVGTD00,NRVDECO2");
 
     } else if (device_driver)
-        addLoader("DEVICEENTRY,DEVICEENTRY2", nullptr);
+        addLoader("DEVICEENTRY,DEVICEENTRY2");
 
     addLoader("EXEENTRY", M_IS_LZMA(ph.method) && device_driver ? "LONGSUB" : "SHORTSUB",
               "JNCDOCOPY", relocsize ? "EXERELPU" : "", "EXEMAIN4",
               M_IS_LZMA(ph.method) ? "" : "EXEMAIN4B", "EXEMAIN4C",
               M_IS_LZMA(ph.method) ? "COMPRESSED_LZMA_START,COMPRESSED_LZMA" : "",
-              "+G5DXXXX,UPX1HEAD,EXECUTPO", nullptr);
+              "+G5DXXXX,UPX1HEAD,EXECUTPO");
     if (ph.method == M_NRV2B_8)
         addLoader("NRV2B16S", // decompressor
                   ph.u_len > DI_LIMIT ? "N2B64K01" : "", "NRV2BEX1",
-                  opt->cpu == opt->CPU_8086 ? "N2BX8601" : "N2B28601", "NRV2BEX2",
-                  opt->cpu == opt->CPU_8086 ? "N2BX8602" : "N2B28602", "NRV2BEX3",
-                  ph.c_len > 0xffff ? "N2B64K02" : "", "NRV2BEX9", nullptr);
+                  opt->cpu_x86 == opt->CPU_8086 ? "N2BX8601" : "N2B28601", "NRV2BEX2",
+                  opt->cpu_x86 == opt->CPU_8086 ? "N2BX8602" : "N2B28602", "NRV2BEX3",
+                  ph.c_len > 0xffff ? "N2B64K02" : "", "NRV2BEX9");
     else if (ph.method == M_NRV2D_8)
         addLoader("NRV2D16S", ph.u_len > DI_LIMIT ? "N2D64K01" : "", "NRV2DEX1",
-                  opt->cpu == opt->CPU_8086 ? "N2DX8601" : "N2D28601", "NRV2DEX2",
-                  opt->cpu == opt->CPU_8086 ? "N2DX8602" : "N2D28602", "NRV2DEX3",
-                  ph.c_len > 0xffff ? "N2D64K02" : "", "NRV2DEX9", nullptr);
+                  opt->cpu_x86 == opt->CPU_8086 ? "N2DX8601" : "N2D28601", "NRV2DEX2",
+                  opt->cpu_x86 == opt->CPU_8086 ? "N2DX8602" : "N2D28602", "NRV2DEX3",
+                  ph.c_len > 0xffff ? "N2D64K02" : "", "NRV2DEX9");
     else if (ph.method == M_NRV2E_8)
         addLoader("NRV2E16S", ph.u_len > DI_LIMIT ? "N2E64K01" : "", "NRV2EEX1",
-                  opt->cpu == opt->CPU_8086 ? "N2EX8601" : "N2E28601", "NRV2EEX2",
-                  opt->cpu == opt->CPU_8086 ? "N2EX8602" : "N2E28602", "NRV2EEX3",
-                  ph.c_len > 0xffff ? "N2E64K02" : "", "NRV2EEX9", nullptr);
+                  opt->cpu_x86 == opt->CPU_8086 ? "N2EX8601" : "N2E28601", "NRV2EEX2",
+                  opt->cpu_x86 == opt->CPU_8086 ? "N2EX8602" : "N2E28602", "NRV2EEX3",
+                  ph.c_len > 0xffff ? "N2E64K02" : "", "NRV2EEX9");
     else if M_IS_LZMA (ph.method)
         return;
     else
@@ -220,28 +217,25 @@ int PackExe::readFileHeader() {
     if (ih.ident != 'M' + 'Z' * 256 && ih.ident != 'Z' + 'M' * 256)
         return 0;
     ih_exesize = ih.m512 + ih.p512 * 512 - (ih.m512 ? 512 : 0);
-    if (!ih_exesize) {
+    if (ih_exesize == 0)
         ih_exesize = file_size;
-    }
     ih_imagesize = ih_exesize - ih.headsize16 * 16;
     ih_overlay = file_size - ih_exesize;
     if (file_size_u < sizeof(ih) || ((ih.m512 | ih.p512) && ih.m512 + ih.p512 * 512u < sizeof(ih)))
         throwCantPack("illegal exe header");
-    if (file_size_u < ih_exesize || ih_imagesize <= 0 || ih_imagesize > ih_exesize)
+    if (ih_exesize > file_size_u || ih_imagesize < 4 || ih_imagesize > ih_exesize)
         throwCantPack("exe header corrupted");
-#if 0
-    printf("dos/exe header: %d %d %d\n", ih_exesize, ih_imagesize, ih_overlay);
-#endif
+    NO_printf("dos/exe header: %d %d %d\n", ih_exesize, ih_imagesize, ih_overlay);
     return UPX_F_DOS_EXE;
 }
 
 bool PackExe::canPack() {
-    if (fn_has_ext(fi->getName(), "sys"))
+    if (fn_has_ext(fi->getName(), "sys")) // dos/sys
         return false;
     if (!readFileHeader())
         return false;
-    if (file_size < 1024)
-        throwCantPack("file is too small");
+    if (file_size < 1024 || ih_imagesize < 512)
+        throwCantPack("file is too small for dos/exe");
     fi->seek(0x3c, SEEK_SET);
     LE32 offs;
     fi->readx(&offs, sizeof(offs));
@@ -249,7 +243,7 @@ bool PackExe::canPack() {
         if (opt->dos_exe.force_stub)
             opt->overlay = opt->COPY_OVERLAY;
         else
-            throwCantPack("can't pack new-exe");
+            throwCantPack("dos/exe: can't pack new-exe");
     }
     return true;
 }
@@ -258,13 +252,17 @@ bool PackExe::canPack() {
 //
 **************************************************************************/
 
-static unsigned optimize_relocs(upx_byte *b, const unsigned size, const upx_byte *relocs,
-                                const unsigned nrelocs, upx_byte *crel, bool *has_9a) {
+static unsigned optimize_relocs(SPAN_S(byte) image, const unsigned image_size,
+                                SPAN_S(const byte) relocs, const unsigned relocnum,
+                                SPAN_S(byte) crel, bool *has_9a) {
+#if WITH_XSPAN >= 2
+    ptr_check_no_overlap(image.data(image_size), image_size, relocs.data(), relocs.size_bytes(),
+                         crel.data(), crel.size_bytes());
+#endif
     if (opt->exact)
         throwCantPackExact();
 
-    upx_byte *const crel_save = crel;
-    unsigned i;
+    SPAN_S_VAR(byte, const crel_start, crel);
     unsigned seg_high = 0;
 #if 0
     unsigned seg_low = 0xffffffff;
@@ -274,19 +272,19 @@ static unsigned optimize_relocs(upx_byte *b, const unsigned size, const upx_byte
     unsigned linear_high = 0;
 #endif
 
-    // pass 1 - find 0x9a bounds
-    for (i = 0; i < nrelocs; i++) {
+    // pass 1 - find 0x9a bounds in image
+    for (unsigned i = 0; i < relocnum; i++) {
         unsigned addr = get_le32(relocs + 4 * i);
-        if (addr >= size - 1)
+        if (addr >= image_size - 1)
             throwCantPack("unexpected relocation 1");
-        if (addr >= 3 && b[addr - 3] == 0x9a) {
-            unsigned seg = get_le16(b + addr);
+        if (addr >= 3 && image[addr - 3] == 0x9a) {
+            unsigned seg = get_le16(image + addr);
             if (seg > seg_high)
                 seg_high = seg;
 #if 0
             if (seg < seg_low)
                 seg_low = seg;
-            unsigned off = get_le16(b+addr-2);
+            unsigned off = get_le16(image + addr - 2);
             if (off < off_low)
                 off_low = off;
             if (off > off_high)
@@ -308,19 +306,20 @@ static unsigned optimize_relocs(upx_byte *b, const unsigned size, const upx_byte
     crel += 4; // to be filled in later
 
     unsigned ones = 0;
-    unsigned es = 0, di, t;
-    i = 0;
-    do {
+    unsigned es = 0;
+    for (unsigned i = 0; i < relocnum;) {
         unsigned addr = get_le32(relocs + 4 * i);
-        set_le16(crel, di = addr & 0x0f);
+        unsigned di = addr & 0x0f;
+        set_le16(crel + 0, di);
         set_le16(crel + 2, (addr >> 4) - es);
-        es = addr >> 4;
         crel += 4;
+        es = addr >> 4;
 
-        for (++i; i < nrelocs; i++) {
+        for (++i; i < relocnum; i++) {
+            unsigned t;
             addr = get_le32(relocs + 4 * i);
-            // printf ("%x\n",es*16+di);
-            if ((addr - es * 16 > 0xfffe) || (i == nrelocs - 1 && addr - es * 16 > 0xff00)) {
+            NO_printf("%x\n", es * 16 + di);
+            if ((addr - es * 16 > 0xfffe) || (i == relocnum - 1 && addr - es * 16 > 0xff00)) {
                 // segment change
                 t = 1 + (0xffff - di) / 254;
                 memset(crel, 1, t);
@@ -329,9 +328,9 @@ static unsigned optimize_relocs(upx_byte *b, const unsigned size, const upx_byte
                 break;
             }
             unsigned offs = addr - es * 16;
-            if (offs >= 3 && b[es * 16 + offs - 3] == 0x9a && offs > di + 3) {
+            if (offs >= 3 && image[es * 16 + offs - 3] == 0x9a && offs > di + 3) {
                 for (t = di; t < offs - 3; t++)
-                    if (b[es * 16 + t] == 0x9a && get_le16(b + es * 16 + t + 3) <= seg_high)
+                    if (image[es * 16 + t] == 0x9a && get_le16(image + es * 16 + t + 3) <= seg_high)
                         break;
                 if (t == offs - 3) {
                     // code 0: search for 0x9a
@@ -342,26 +341,25 @@ static unsigned optimize_relocs(upx_byte *b, const unsigned size, const upx_byte
                 }
             }
             t = offs - di;
-            if (t < 2)
+            if ((int) t < 2)
                 throwCantPack("unexpected relocation 2");
-
             while (t >= 256) {
                 // code 1: add 254, don't reloc
                 *crel++ = 1;
                 t -= 254;
                 ones++;
             }
-            *crel++ = (unsigned char) t;
+            *crel++ = (byte) t;
             di = offs;
         }
-    } while (i < nrelocs);
+    }
     *crel++ = 1;
     ones++;
-    set_le16(crel_save, ones);
-    set_le16(crel_save + 2, seg_high);
+    set_le16(crel_start, ones);
+    set_le16(crel_start + 2, seg_high);
 
-    // OutputFile::dump("x.rel", crel_save, crel - crel_save);
-    return (unsigned) (crel - crel_save);
+    // OutputFile::dump("x.rel", crel_start, ptr_udiff_bytes(crel, crel_start));
+    return ptr_udiff_bytes(crel, crel_start);
 }
 
 /*************************************************************************
@@ -371,16 +369,14 @@ static unsigned optimize_relocs(upx_byte *b, const unsigned size, const upx_byte
 void PackExe::pack(OutputFile *fo) {
     unsigned ic;
 
-    if (ih.relocs > MAXRELOCS)
+    const unsigned relocnum = ih.relocs;
+    if (relocnum > MAXRELOCSIZE) // early check
         throwCantPack("too many relocations");
     checkOverlay(ih_overlay);
 
-    // alloc buffers
-    relocsize = RSFCRI + 4 * ih.relocs;
-    ibuf.alloc(ih_imagesize + 16 + relocsize + 2);
-    obuf.allocForCompression(ih_imagesize + 16 + relocsize + 2);
-
     // read image
+    // image + space for optimized relocs + safety/alignments
+    ibuf.alloc(ih_imagesize + 4 * relocnum + 1024);
     fi->seek(ih.headsize16 * 16, SEEK_SET);
     fi->readx(ibuf, ih_imagesize);
 
@@ -389,37 +385,41 @@ void PackExe::pack(OutputFile *fo) {
     device_driver = get_le32(ibuf) == 0xffffffffu;
 
     // relocations
+    relocsize = 0;
     has_9a = false;
-    upx_byte *w = ibuf + ih_imagesize;
-    if (ih.relocs) {
-        upx_byte *wr = w + RSFCRI;
-
+    if (relocnum) {
+        MemBuffer mb_relocs(4 * relocnum);
+        SPAN_S_VAR(byte, relocs, mb_relocs);
         fi->seek(ih.relocoffs, SEEK_SET);
-        fi->readx(wr, 4 * ih.relocs);
+        fi->readx(relocs, 4 * relocnum);
 
-        for (ic = 0; ic < ih.relocs; ic++) {
-            unsigned jc = get_le32(wr + 4 * ic);
-            set_le32(wr + 4 * ic, ((jc >> 16) * 16 + (jc & 0xffff)) & 0xfffff);
+        // dos/exe runs in real-mode, so convert to linear addresses
+        for (ic = 0; ic < relocnum; ic++) {
+            unsigned jc = get_le32(relocs + 4 * ic);
+            set_le32(relocs + 4 * ic, ((jc >> 16) * 16 + (jc & 0xffff)) & 0xfffff);
         }
-        qsort(wr, ih.relocs, 4, le32_compare);
-        relocsize = optimize_relocs(ibuf, ih_imagesize, wr, ih.relocs, w, &has_9a);
-        set_le16(w + relocsize, relocsize + 2);
+        qsort(raw_bytes(relocs, 4 * relocnum), relocnum, 4, le32_compare);
+
+        SPAN_S_VAR(byte, image, ibuf + 0, ih_imagesize);
+        SPAN_S_VAR(byte, crel, ibuf + ih_imagesize, ibuf);
+        relocsize = optimize_relocs(image, ih_imagesize, relocs, relocnum, crel, &has_9a);
+        set_le16(crel + relocsize, relocsize + 2);
         relocsize += 2;
-        if (relocsize > MAXRELOCS)
+        assert(relocsize >= 11);
+        if (relocsize > MAXRELOCSIZE) // optimize_relocs did not help
             throwCantPack("too many relocations");
-#if 0
-        upx_byte out[9*relocsize/8+1024];
-        unsigned in_len = relocsize;
-        unsigned out_len = 0;
-        ucl_nrv2b_99_compress(w, in_len, out, &out_len, nullptr, 9, nullptr, nullptr);
-        printf("reloc compress: %d -> %d\n", in_len, out_len);
+#if TESTING && 0
+        unsigned rout_len = MemBuffer::getSizeForCompression(relocsize);
+        MemBuffer rout(rout_len);
+        ucl_nrv2b_99_compress(raw_bytes(crel, relocsize), relocsize, rout, &rout_len, nullptr, 9,
+                              nullptr, nullptr);
+        printf("dos/exe reloc compress: %d -> %d\n", relocsize, rout_len);
 #endif
-    } else {
-        relocsize = 0;
     }
 
     // prepare packheader
     ph.u_len = ih_imagesize + relocsize;
+    obuf.allocForCompression(ph.u_len);
     // prepare filter
     Filter ft(ph.level);
     // compress (max_match = 8192)
@@ -467,7 +467,7 @@ void PackExe::pack(OutputFile *fo) {
     oh.max = ic < 0xffff && ih.max != 0xffff ? ic : 0xffff;
 
     // set extra info
-    unsigned char extra_info[9];
+    byte extra_info[9];
     unsigned eisize = 0;
     if (oh.ss != ih.ss) {
         set_le16(extra_info + eisize, ih.ss);
@@ -489,7 +489,7 @@ void PackExe::pack(OutputFile *fo) {
         eisize += 2;
         flag |= MAXMEM;
     }
-    extra_info[eisize++] = (unsigned char) flag;
+    extra_info[eisize++] = (byte) flag;
 
     if (M_IS_NRV2B(ph.method) || M_IS_NRV2D(ph.method) || M_IS_NRV2E(ph.method))
         linker->defineSymbol("bx_magic", 0x7FFF + 0x10 * ((packedsize & 15) + 1));
@@ -511,9 +511,7 @@ void PackExe::pack(OutputFile *fo) {
         oh.firstreloc = ih.cs * 0x10000 + ih.ip;
     }
 
-    // g++ 3.1 does not like the following line...
     oh.relocoffs = offsetof(exe_header_t, firstreloc);
-    // oh.relocoffs = ptr_udiff_bytes(&oh.firstreloc, &oh);
 
     linker->defineSymbol("destination_segment", oh.ss - ph.c_len / 16 - e_len / 16);
     linker->defineSymbol("source_segment", e_len / 16 + (copysize - firstcopy) / 16);
@@ -526,7 +524,7 @@ void PackExe::pack(OutputFile *fo) {
     linker->defineSymbol("attribute", get_le16(ibuf + 4));
     linker->defineSymbol("orig_strategy", get_le16(ibuf + 6));
 
-    const unsigned outputlen = sizeof(oh) + lsize + packedsize + eisize;
+    const unsigned outputlen = sizeof(oh) + e_len + packedsize + d_len + eisize;
     oh.m512 = outputlen & 511;
     oh.p512 = (outputlen + 511) >> 9;
 
@@ -538,35 +536,32 @@ void PackExe::pack(OutputFile *fo) {
     memcpy(loader, getLoader(), lsize);
     patchPackHeader(loader, e_len);
 
-    NO_fprintf(stderr, "\ne_len=%x d_len=%x c_len=%x oo=%x ulen=%x destp=%x copys=%x images=%x",
-               e_len, d_len, packedsize, ph.overlap_overhead, ph.u_len, /*destpara*/ 0, copysize,
-               ih_imagesize);
+    NO_fprintf(stderr, "\ne_len=%x d_len=%x c_len=%x oo=%x ulen=%x copysize=%x imagesize=%x", e_len,
+               d_len, packedsize, ph.overlap_overhead, ph.u_len, copysize, ih_imagesize);
 
     // write header + write loader + compressed file
 #if TESTING
     if (opt->debug.debug_level)
         printf("\n%d %d %d %d\n", (int) sizeof(oh), e_len, packedsize, d_len);
 #endif
-    fo->write(&oh, sizeof(oh));
-    fo->write(loader, e_len); // entry
-    fo->write(obuf, packedsize);
-    fo->write(loader + e_len, d_len); // decompressor
-    fo->write(extra_info, eisize);
+    fo->write(&oh, sizeof(oh));       // program header
+    fo->write(loader, e_len);         // entry code
+    fo->write(obuf, packedsize);      // compressed data
+    fo->write(loader + e_len, d_len); // decompressor code
+    fo->write(extra_info, eisize);    // extra info for unpacking
     assert(eisize <= 9);
-#if 0
-    printf("%-13s: program hdr  : %8ld bytes\n", getName(), (long) sizeof(oh));
-    printf("%-13s: entry        : %8ld bytes\n", getName(), (long) e_len);
-    printf("%-13s: compressed   : %8ld bytes\n", getName(), (long) packedsize);
-    printf("%-13s: decompressor : %8ld bytes\n", getName(), (long) d_len);
-    printf("%-13s: extra info   : %8ld bytes\n", getName(), (long) eisize);
-#endif
+    NO_printf("%-13s: program hdr  : %8u bytes\n", getName(), usizeof(oh));
+    NO_printf("%-13s: entry        : %8u bytes\n", getName(), e_len);
+    NO_printf("%-13s: compressed   : %8u bytes\n", getName(), packedsize);
+    NO_printf("%-13s: decompressor : %8u bytes\n", getName(), d_len);
+    NO_printf("%-13s: extra info   : %8u bytes\n", getName(), eisize);
 
     // verify
     verifyOverlappingDecompression();
 
     // copy the overlay
     copyOverlay(fo, ih_overlay, obuf);
-    // fprintf (stderr,"%x %x\n",relocsize,ph.u_len);
+    NO_fprintf(stderr, "dos/exe %x %x\n", relocsize, ph.u_len);
 
     // finally check the compression ratio
     if (!checkFinalCompressionRatio(fo))
@@ -609,41 +604,46 @@ void PackExe::unpack(OutputFile *fo) {
     decompress(ibuf + e_len, obuf);
 
     unsigned imagesize = ih_imagesize;
-    imagesize--;
-    const unsigned char flag = ibuf[imagesize];
+    imagesize -= 1;
+    const byte flag = ibuf[imagesize];
 
-    unsigned relocn = 0;
-    SPAN_S_VAR(upx_byte, relocs, obuf + ph.u_len, obuf);
-
-    MemBuffer mb_wrkmem;
-    SPAN_0_VAR(upx_byte, wrkmem, nullptr);
+    // relocations
+    unsigned relocnum = 0;
+    SPAN_S_VAR(const byte, relocstart, obuf + ph.u_len, obuf);
+    MemBuffer mb_relocs;
+    SPAN_0_VAR(byte, relocs, nullptr);
     if (!(flag & NORELOC)) {
-        relocs -= get_le16(obuf + (ph.u_len - 2));
+        mb_relocs.alloc(4 * MAXRELOCSIZE);
+        relocs = mb_relocs; // => now a SPAN_S
+
+        relocsize = get_le16(obuf + ph.u_len - 2);
         ph.u_len -= 2;
+        if (relocsize < 11 || relocsize > MAXRELOCSIZE || relocsize >= imagesize)
+            throwCantUnpack("bad relocations");
+        relocstart -= relocsize;
 
-        mb_wrkmem.alloc(4 * MAXRELOCS);
-        wrkmem = mb_wrkmem; // => now a SPAN_S
-        unsigned es = 0, ones = get_le16(relocs);
-        const unsigned seghi = get_le16(relocs + 2);
-        SPAN_S_VAR(const upx_byte, p, relocs + 4);
-
+        // unoptimize_relocs
+        unsigned ones = get_le16(relocstart);
+        const unsigned seg_high = get_le16(relocstart + 2);
+        SPAN_S_VAR(const byte, p, relocstart + 4);
+        unsigned es = 0;
         while (ones) {
             unsigned di = get_le16(p);
             es += get_le16(p + 2);
             bool dorel = true;
             for (p += 4; ones && di < 0x10000; p++) {
                 if (dorel) {
-                    set_le16(wrkmem + (4 * relocn), di);
-                    set_le16(wrkmem + (2 + 4 * relocn++), es);
-                    NO_printf("%x\n", es * 16 + di);
+                    set_le16(relocs + (4 * relocnum + 0), di);
+                    set_le16(relocs + (4 * relocnum + 2), es);
+                    NO_printf("dos/exe unreloc %4d %6x\n", relocnum, es * 16 + di);
+                    relocnum++;
                 }
                 dorel = true;
                 if (*p == 0) {
-                    SPAN_S_VAR(const upx_byte, q, obuf);
-                    for (q = obuf + (es * 16 + di); !(*q == 0x9a && get_le16(q + 3) <= seghi);
-                         q++) {
-                    }
-                    di = ptr_diff_bytes(q, obuf + (es * 16)) + 3;
+                    SPAN_S_VAR(const byte, q, obuf + (es * 16 + di), obuf);
+                    while (!(*q == 0x9a && get_le16(q + 3) <= seg_high))
+                        q++;
+                    di = ptr_udiff_bytes(q, obuf + (es * 16)) + 3;
                 } else if (*p == 1) {
                     di += 254;
                     if (di < 0x10000)
@@ -656,19 +656,19 @@ void PackExe::unpack(OutputFile *fo) {
     }
 
     // fill new exe header
-    memset(&oh, 0, sizeof(oh));
+    mem_clear(&oh);
     oh.ident = 'M' + 'Z' * 256;
 
-    if (relocn) {
-        oh.relocs = relocn;
-        while (relocn & 3)
-            set_le32(wrkmem + (4 * relocn++), 0);
+    if (relocnum) {
+        oh.relocs = relocnum;
+        while (relocnum & 3) // paragraph align
+            set_le32(relocs + (4 * relocnum++), 0);
     }
 
-    unsigned outputlen = ptr_udiff_bytes(relocs, obuf) + sizeof(oh) + relocn * 4;
+    unsigned outputlen = sizeof(oh) + 4 * relocnum + ptr_udiff_bytes(relocstart, obuf);
     oh.m512 = outputlen & 511;
     oh.p512 = (outputlen + 511) >> 9;
-    oh.headsize16 = 2 + relocn / 4;
+    oh.headsize16 = 2 + relocnum / 4;
 
     oh.max = ih.max;
     oh.min = ih.min;
@@ -703,16 +703,14 @@ void PackExe::unpack(OutputFile *fo) {
 
     // write header + relocations + uncompressed file
     fo->write(&oh, sizeof(oh));
-    if (relocn)
-        fo->write(wrkmem, relocn * 4);
-    fo->write(obuf, ptr_udiff_bytes(relocs, obuf));
+    if (relocnum)
+        fo->write(relocs, 4 * relocnum);
+    fo->write(obuf, ptr_udiff_bytes(relocstart, obuf));
 
     // copy the overlay
     copyOverlay(fo, ih_overlay, obuf);
 }
 
-Linker *PackExe::newLinker() const { return new ElfLinkerX86(); }
-
 /*
 
 memory layout at decompression time
