@@ -26,6 +26,7 @@
  */
 
 
+#define ALLOW_INT_PLUS_MEMBUFFER 1
 #include "conf.h"
 
 #include "p_elf.h"
@@ -594,7 +595,7 @@ void PackBvmlinuzI386::pack(OutputFile *fo)
             (res->lit_context_bits << 0) |
             (res->lit_pos_bits << 8) |
             (res->pos_bits << 16);
-        if (linker->bele->isBE()) // big endian - bswap32
+        if (bele->isBE()) // big endian - bswap32
             properties = bswap32(properties);
         linker->defineSymbol("lzma_properties", properties);
         // -2 for properties
@@ -749,7 +750,7 @@ int PackVmlinuzARMEL::readFileHeader()
             return 0;
         }
     }
-    return UPX_F_VMLINUZ_ARMEL;
+    return UPX_F_VMLINUZ_ARM;
 }
 
 int PackVmlinuzARMEL::decompressKernel()
