@@ -25,9 +25,8 @@
    <markus@oberhumer.com>               <ezerotven+github@gmail.com>
  */
 
-#include "../conf.h"
-#include "util.h"
-
+#include "../headers.h"
+#include <algorithm>
 #define ACC_WANT_ACC_INCI_H 1
 #include "../miniacc.h"
 #define ACC_WANT_ACCLIB_GETOPT 1
@@ -37,6 +36,7 @@
 #define ACC_WANT_ACCLIB_WILDARGV 1
 #undef HAVE_MKDIR
 #include "../miniacc.h"
+#include "../conf.h"
 
 /*************************************************************************
 // assert sane memory buffer sizes to protect against integer overflows
@@ -46,13 +46,16 @@
 
 ACC_COMPILE_TIME_ASSERT_HEADER(UPX_RSIZE_MAX_MEM == UPX_RSIZE_MAX)
 ACC_COMPILE_TIME_ASSERT_HEADER(UPX_RSIZE_MAX_STR <= UPX_RSIZE_MAX / 256)
-ACC_COMPILE_TIME_ASSERT_HEADER(2ull * UPX_RSIZE_MAX * 9 / 8 + 16 * 1024 * 1024 < INT_MAX)
-ACC_COMPILE_TIME_ASSERT_HEADER(UPX_RSIZE_MAX >= 65536 * 8192)
+ACC_COMPILE_TIME_ASSERT_HEADER(2ull * UPX_RSIZE_MAX * 9 / 8 + 256 * 1024 * 1024 < INT_MAX)
+ACC_COMPILE_TIME_ASSERT_HEADER(2ull * UPX_RSIZE_MAX * 10 / 8 + 128 * 1024 * 1024 <= INT_MAX + 1u)
+ACC_COMPILE_TIME_ASSERT_HEADER(5ull * UPX_RSIZE_MAX < UINT_MAX)
+ACC_COMPILE_TIME_ASSERT_HEADER(UPX_RSIZE_MAX >= 8192 * 65536)
+ACC_COMPILE_TIME_ASSERT_HEADER(UPX_RSIZE_MAX_STR >= 1024)
 
 upx_rsize_t mem_size(upx_uint64_t element_size, upx_uint64_t n, upx_uint64_t extra1,
                      upx_uint64_t extra2) {
     assert(element_size > 0);
-    if very_unlikely (element_size > UPX_RSIZE_MAX)
+    if very_unlikely (element_size == 0 || element_size > UPX_RSIZE_MAX)
         throwCantPack("mem_size 1; take care");
     if very_unlikely (n > UPX_RSIZE_MAX)
         throwCantPack("mem_size 2; take care");
@@ -68,8 +71,8 @@ upx_rsize_t mem_size(upx_uint64_t element_size, upx_uint64_t n, upx_uint64_t ext
 
 bool mem_size_valid(upx_uint64_t element_size, upx_uint64_t n, upx_uint64_t extra1,
                     upx_uint64_t extra2) noexcept {
-    assert(element_size > 0);
-    if very_unlikely (element_size > UPX_RSIZE_MAX)
+    assert_noexcept(element_size > 0);
+    if very_unlikely (element_size == 0 || element_size > UPX_RSIZE_MAX)
         return false;
     if very_unlikely (n > UPX_RSIZE_MAX)
         return false;
@@ -98,6 +101,10 @@ TEST_CASE("mem_size") {
     CHECK_THROWS(mem_size(1, 0x30000000, 0x30000000, 0x30000000));
 }
 
+/*************************************************************************
+// ptr util
+**************************************************************************/
+
 int ptr_diff_bytes(const void *a, const void *b) {
     if very_unlikely (a == nullptr) {
         throwCantPack("ptr_diff_bytes null 1; take care");
@@ -105,13 +112,13 @@ int ptr_diff_bytes(const void *a, const void *b) {
     if very_unlikely (b == nullptr) {
         throwCantPack("ptr_diff_bytes null 2; take care");
     }
-    ptrdiff_t d = (const char *) a - (const char *) b;
+    ptrdiff_t d = (const charptr) a - (const charptr) b;
     if (a >= b) {
         if very_unlikely (!mem_size_valid_bytes(d))
-            throwCantPack("ptr_diff_bytes 1; take care");
+            throwCantPack("ptr_diff_bytes-1; take care");
     } else {
-        if very_unlikely (!mem_size_valid_bytes(-d))
-            throwCantPack("ptr_diff_bytes 2; take care");
+        if very_unlikely (!mem_size_valid_bytes(0ll - d))
+            throwCantPack("ptr_diff_bytes-2; take care");
     }
     return ACC_ICONV(int, d);
 }
@@ -124,7 +131,7 @@ unsigned ptr_udiff_bytes(const void *a, const void *b) {
 }
 
 TEST_CASE("ptr_diff") {
-    char buf[4] = {0, 1, 2, 3};
+    byte buf[4] = {0, 1, 2, 3};
     CHECK_THROWS(ptr_diff_bytes(nullptr, buf));
     CHECK_THROWS(ptr_diff_bytes(buf, nullptr));
     CHECK(ptr_diff(buf, buf) == 0);
@@ -135,19 +142,206 @@ TEST_CASE("ptr_diff") {
     CHECK_THROWS(ptr_udiff(buf, buf + 1));
 }
 
+// check that 2 buffers do not overlap; will throw on error
+void uintptr_check_no_overlap(upx_uintptr_t a, size_t a_size, upx_uintptr_t b, size_t b_size) {
+    if very_unlikely (a == 0 || b == 0)
+        throwCantPack("ptr_check_no_overlap-nullptr");
+    upx_uintptr_t a_end = a + mem_size(1, a_size);
+    upx_uintptr_t b_end = b + mem_size(1, b_size);
+    if very_unlikely (a_end < a || b_end < b) // wrap-around
+        throwCantPack("ptr_check_no_overlap-overflow");
+    // same as (!(a >= b_end || b >= a_end))
+    // same as (!(a_end <= b || b_end <= a))
+    if very_unlikely (a < b_end && b < a_end)
+        throwCantPack("ptr_check_no_overlap-ab");
+}
+
+// check that 3 buffers do not overlap; will throw on error
+void uintptr_check_no_overlap(upx_uintptr_t a, size_t a_size, upx_uintptr_t b, size_t b_size,
+                              upx_uintptr_t c, size_t c_size) {
+    if very_unlikely (a == 0 || b == 0 || c == 0)
+        throwCantPack("ptr_check_no_overlap-nullptr");
+    upx_uintptr_t a_end = a + mem_size(1, a_size);
+    upx_uintptr_t b_end = b + mem_size(1, b_size);
+    upx_uintptr_t c_end = c + mem_size(1, c_size);
+    if very_unlikely (a_end < a || b_end < b || c_end < c) // wrap-around
+        throwCantPack("ptr_check_no_overlap-overflow");
+    if very_unlikely (a < b_end && b < a_end)
+        throwCantPack("ptr_check_no_overlap-ab");
+    if very_unlikely (a < c_end && c < a_end)
+        throwCantPack("ptr_check_no_overlap-ac");
+    if very_unlikely (b < c_end && c < b_end)
+        throwCantPack("ptr_check_no_overlap-bc");
+}
+
+#if DEBUG
+TEST_CASE("ptr_check_no_overlap 2") {
+    byte p[4] = {};
+
+    auto check_nothrow = [&p](int a, int as, int b, int bs) {
+        CHECK_NOTHROW(ptr_check_no_overlap(p + a, as, p + b, bs)); // ab
+        CHECK_NOTHROW(ptr_check_no_overlap(p + b, bs, p + a, as)); // ba
+    };
+    auto check_throws_ = [&p](int a, int as, int b, int bs) {
+        CHECK_THROWS(ptr_check_no_overlap(p + a, as, p + b, bs)); // ab
+        CHECK_THROWS(ptr_check_no_overlap(p + b, bs, p + a, as)); // ba
+    };
+
+    check_throws_(0, 1, 0, 1);
+    check_nothrow(0, 1, 1, 1);
+    check_throws_(0, 2, 1, 1);
+    check_nothrow(0, 2, 2, 1);
+    // empty buffers at edge
+    check_nothrow(0, 0, 0, 0);
+    check_nothrow(0, 0, 0, 1);
+    check_nothrow(0, 0, 1, 0);
+    // empty buffer
+    check_nothrow(0, 4, 0, 0);
+    check_throws_(0, 4, 1, 0);
+    check_throws_(0, 4, 2, 0);
+    check_throws_(0, 4, 3, 0);
+    check_nothrow(0, 4, 4, 0);
+}
+
+TEST_CASE("ptr_check_no_overlap 3") {
+    byte p[4] = {};
+
+    auto check_nothrow = [&p](int a, int as, int b, int bs, int c, int cs) {
+        CHECK_NOTHROW(ptr_check_no_overlap(p + a, as, p + b, bs, p + c, cs)); // abc
+        CHECK_NOTHROW(ptr_check_no_overlap(p + a, as, p + c, cs, p + b, bs)); // acb
+        CHECK_NOTHROW(ptr_check_no_overlap(p + b, bs, p + a, as, p + c, cs)); // bac
+        CHECK_NOTHROW(ptr_check_no_overlap(p + b, bs, p + c, cs, p + a, as)); // bca
+        CHECK_NOTHROW(ptr_check_no_overlap(p + c, cs, p + a, as, p + b, bs)); // cab
+        CHECK_NOTHROW(ptr_check_no_overlap(p + c, cs, p + b, bs, p + a, as)); // cba
+    };
+    auto check_throws_ = [&p](int a, int as, int b, int bs, int c, int cs) {
+        CHECK_THROWS(ptr_check_no_overlap(p + a, as, p + b, bs, p + c, cs)); // abc
+        CHECK_THROWS(ptr_check_no_overlap(p + a, as, p + c, cs, p + b, bs)); // acb
+        CHECK_THROWS(ptr_check_no_overlap(p + b, bs, p + a, as, p + c, cs)); // bac
+        CHECK_THROWS(ptr_check_no_overlap(p + b, bs, p + c, cs, p + a, as)); // bca
+        CHECK_THROWS(ptr_check_no_overlap(p + c, cs, p + a, as, p + b, bs)); // cab
+        CHECK_THROWS(ptr_check_no_overlap(p + c, cs, p + b, bs, p + a, as)); // cba
+    };
+
+    check_throws_(0, 1, 0, 1, 1, 1);
+    check_nothrow(0, 1, 1, 1, 2, 1);
+    check_throws_(0, 2, 1, 1, 2, 1);
+    check_nothrow(0, 2, 2, 1, 3, 1);
+    // empty buffers at edge
+    check_nothrow(0, 0, 0, 0, 0, 0);
+    check_nothrow(0, 0, 0, 0, 0, 1);
+    check_nothrow(0, 0, 0, 1, 1, 1);
+    check_nothrow(0, 0, 1, 0, 1, 1);
+    // empty buffer
+    check_nothrow(0, 4, 0, 0, 0, 0);
+    check_throws_(0, 4, 1, 0, 0, 0);
+    check_throws_(0, 4, 2, 0, 0, 0);
+    check_throws_(0, 4, 3, 0, 0, 0);
+    check_nothrow(0, 4, 4, 0, 0, 0);
+    // empty buffer
+    check_throws_(0, 4, 0, 0, 1, 0);
+    check_throws_(0, 4, 1, 0, 1, 0);
+    check_throws_(0, 4, 2, 0, 1, 0);
+    check_throws_(0, 4, 3, 0, 1, 0);
+    check_throws_(0, 4, 4, 0, 1, 0);
+}
+#endif // DEBUG
+
 /*************************************************************************
-// bele.h
+// stdlib
 **************************************************************************/
 
-namespace N_BELE_CTP {
-const BEPolicy be_policy;
-const LEPolicy le_policy;
-} // namespace N_BELE_CTP
+void *upx_calloc(size_t n, size_t element_size) {
+    size_t bytes = mem_size(element_size, n); // assert size
+    void *p = malloc(bytes);
+    if (p != nullptr)
+        memset(p, 0, bytes);
+    return p;
+}
 
-namespace N_BELE_RTP {
-const BEPolicy be_policy;
-const LEPolicy le_policy;
-} // namespace N_BELE_RTP
+// simple unoptimized memswap()
+void upx_memswap(void *a, void *b, size_t n) {
+    if (a != b && n != 0) {
+        char *x = (char *) a;
+        char *y = (char *) b;
+        do {
+            // strange clang-analyzer-15 false positive when compiling in Debug mode
+            // clang-analyzer-core.uninitialized.Assign
+            char tmp = *x; // NOLINT(*core.uninitialized.Assign) // bogus clang-analyzer warning
+            *x++ = *y;
+            *y++ = tmp;
+        } while (--n != 0);
+    }
+}
+
+// extremely simple (and beautiful) stable sort: Gnomesort
+// WARNING: O(n^2) and thus very inefficient for large n
+void upx_stable_sort(void *array, size_t n, size_t element_size,
+                     int (*compare)(const void *, const void *)) {
+    for (size_t i = 1; i < n; i++) {
+        char *a = (char *) array + element_size * i;        // a := &array[i]
+        if (i != 0 && compare(a - element_size, a) > 0) {   // if a[-1] > a[0] then
+            upx_memswap(a - element_size, a, element_size); //   swap elements a[-1] <=> a[0]
+            i -= 2;                                         //   and decrease i
+        }
+    }
+}
+
+#if DEBUG
+TEST_CASE("upx_stable_sort") {
+    {
+        unsigned a[] = {0, 1};
+        upx_stable_sort(a, 2, sizeof(*a), ne32_compare);
+        CHECK((a[0] == 0 && a[1] == 1));
+    }
+    {
+        unsigned a[] = {1, 0};
+        upx_stable_sort(a, 2, sizeof(*a), ne32_compare);
+        CHECK((a[0] == 0 && a[1] == 1));
+    }
+    {
+        BE64 a[3];
+        a[0] = 257;
+        a[1] = 256;
+        a[2] = 255;
+        upx_stable_sort(a, 3, sizeof(*a), be64_compare);
+        CHECK((a[0] == 255 && a[1] == 256 && a[2] == 257));
+    }
+}
+
+#if __cplusplus >= 202002L // use C++20 std::next_permutation() to test all permutations
+namespace {
+struct TestSortAllPermutations {
+    static upx_uint64_t test(size_t n) {
+        constexpr size_t N = 16;
+        assert(n > 0 && n <= N);
+        LE16 perm[N];
+        for (size_t i = 0; i < n; i++)
+            perm[i] = 255 + i;
+        upx_uint64_t num_perms = 0;
+        do {
+            LE16 a[N];
+            memcpy(a, perm, sizeof(*a) * n);
+            upx_stable_sort(a, n, sizeof(*a), le16_compare);
+            for (size_t i = 0; i < n; i++)
+                assert((a[i] == 255 + i));
+            num_perms += 1;
+        } while (std::next_permutation(perm, perm + n));
+        return num_perms;
+    }
+};
+} // namespace
+TEST_CASE("upx_stable_sort") {
+    CHECK(TestSortAllPermutations::test(1) == 1);
+    CHECK(TestSortAllPermutations::test(2) == 2);
+    CHECK(TestSortAllPermutations::test(3) == 6);
+    CHECK(TestSortAllPermutations::test(4) == 24);
+    CHECK(TestSortAllPermutations::test(5) == 120);
+    // CHECK(TestSortAllPermutations::test(6) == 720);
+    // CHECK(TestSortAllPermutations::test(7) == 5040);
+}
+#endif // C++20
+#endif // DEBUG
 
 /*************************************************************************
 // qsort() util
@@ -253,13 +447,13 @@ int __acc_cdecl_qsort le64_compare_signed(const void *e1, const void *e2) {
 // find and mem_replace util
 **************************************************************************/
 
-int find(const void *buf, int blen, const void *what, int wlen) {
+int find(const void *buf, int blen, const void *what, int wlen) noexcept {
     // nullptr is explicitly allowed here
-    if (buf == nullptr || blen <= 0 || what == nullptr || wlen <= 0)
+    if (buf == nullptr || blen < wlen || what == nullptr || wlen <= 0)
         return -1;
 
-    const unsigned char *b = (const unsigned char *) buf;
-    unsigned char first_byte = *(const unsigned char *) what;
+    const byte *b = (const byte *) buf;
+    byte first_byte = *(const byte *) what;
 
     blen -= wlen;
     for (int i = 0; i <= blen; i++, b++)
@@ -269,45 +463,45 @@ int find(const void *buf, int blen, const void *what, int wlen) {
     return -1;
 }
 
-int find_be16(const void *b, int blen, unsigned what) {
-    unsigned char w[2];
+int find_be16(const void *b, int blen, unsigned what) noexcept {
+    byte w[2];
     set_be16(w, what);
     return find(b, blen, w, 2);
 }
 
-int find_be32(const void *b, int blen, unsigned what) {
-    unsigned char w[4];
+int find_be32(const void *b, int blen, unsigned what) noexcept {
+    byte w[4];
     set_be32(w, what);
     return find(b, blen, w, 4);
 }
 
-int find_be64(const void *b, int blen, upx_uint64_t what) {
-    unsigned char w[8];
+int find_be64(const void *b, int blen, upx_uint64_t what) noexcept {
+    byte w[8];
     set_be64(w, what);
     return find(b, blen, w, 8);
 }
 
-int find_le16(const void *b, int blen, unsigned what) {
-    unsigned char w[2];
+int find_le16(const void *b, int blen, unsigned what) noexcept {
+    byte w[2];
     set_le16(w, what);
     return find(b, blen, w, 2);
 }
 
-int find_le32(const void *b, int blen, unsigned what) {
-    unsigned char w[4];
+int find_le32(const void *b, int blen, unsigned what) noexcept {
+    byte w[4];
     set_le32(w, what);
     return find(b, blen, w, 4);
 }
 
-int find_le64(const void *b, int blen, upx_uint64_t what) {
-    unsigned char w[8];
+int find_le64(const void *b, int blen, upx_uint64_t what) noexcept {
+    byte w[8];
     set_le64(w, what);
     return find(b, blen, w, 8);
 }
 
 TEST_CASE("find") {
     CHECK(find(nullptr, -1, nullptr, -1) == -1);
-    static const unsigned char b[16] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15};
+    static const byte b[16] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15};
     CHECK(find(b, 16, b, 0) == -1);
     for (int i = 0; i < 16; i++) {
         CHECK(find(b, 16, b + i, 1) == i);
@@ -326,8 +520,8 @@ TEST_CASE("find") {
     CHECK(find_le64(b, 15, 0x0f0e0d0c0b0a0908ULL) == -1);
 }
 
-int mem_replace(void *buf, int blen, const void *what, int wlen, const void *replacement) {
-    unsigned char *b = (unsigned char *) buf;
+int mem_replace(void *buf, int blen, const void *what, int wlen, const void *replacement) noexcept {
+    byte *b = (byte *) buf;
     int boff = 0;
     int n = 0;
 
@@ -353,6 +547,20 @@ TEST_CASE("mem_replace") {
     CHECK(strcmp(b, "cdcdcdcdefgefghh") == 0);
 }
 
+/*************************************************************************
+// bele.h globals
+**************************************************************************/
+
+namespace N_BELE_CTP {
+const BEPolicy be_policy;
+const LEPolicy le_policy;
+} // namespace N_BELE_CTP
+
+namespace N_BELE_RTP {
+const BEPolicy be_policy;
+const LEPolicy le_policy;
+} // namespace N_BELE_RTP
+
 /*************************************************************************
 // fn - FileName util
 **************************************************************************/
@@ -364,7 +572,7 @@ static const char dir_sep[] = "/\\";
 #define fn_is_drive(s) (s[0] && s[1] == ':')
 #define fn_is_sep(c) (strchr(dir_sep, c) != nullptr)
 #define fn_skip_drive(s) (fn_is_drive(s) ? (s) + 2 : (s))
-#define fn_tolower(c) (tolower(((unsigned char) (c))))
+#define fn_tolower(c) (tolower(((uchar) (c))))
 
 #else
 
@@ -468,14 +676,14 @@ bool file_exists(const char *name) {
     }
 
     /* return true if we can stat it */
-    // memset(&st, 0, sizeof(st));
+    // mem_clear(&st);
     r = stat(name, &st);
     if (r != -1)
         return true;
 
 /* return true if we can lstat it */
-#if (HAVE_LSTAT)
-    // memset(&st, 0, sizeof(st));
+#if HAVE_LSTAT
+    // mem_clear(&st);
     r = lstat(name, &st);
     if (r != -1)
         return true;
@@ -552,7 +760,7 @@ bool makebakname(char *ofilename, size_t size, const char *ifilename, bool force
 **************************************************************************/
 
 unsigned get_ratio(upx_uint64_t u_len, upx_uint64_t c_len) {
-    const unsigned n = 1000 * 1000;
+    constexpr unsigned n = 1000 * 1000;
     if (u_len == 0)
         return c_len == 0 ? 0 : n;
     upx_uint64_t x = c_len * n;
